/************************************************/
/*                                              */
/*  definition file for AR B2201 hardware V1.0  */    
/*                                              */
/************************************************/

unsigned short card_address;
BYTE    card_buf_addr_low = 0x00;
BYTE    card_buf_addr_high = 0x00;
BYTE    card_control_byte = 0xFF;

/* Configure AR B2201 I/O card
         channel 0:  OUTPUT          74x273  ADDR_LOW
         channel 1:  INPUT / OUTPUT  74x374  DATA_CHANNEL
         channel 2:  OUTPUT          74x273  ADDR_HIGH
         channel 3:  OUTPUT          74x273  CONTROL
                        0: /WR
                        1: /RD
                        2: /MREQ
                        3: /RS
                        4: /WR_FLASH
			6: /LED
			7: /channel_1_output
         channel 4:  INPUT
         channel 5:  INPUT
         channel 6:  INPUT
         channel 7:  INPUT
*/

#define B2201		((unsigned short) (card_address))
#define B2201_CHAN0	((unsigned short) (card_address))
#define B2201_CHAN1	((unsigned short) (card_address+1))
#define B2201_CHAN2	((unsigned short) (card_address+2))
#define B2201_CHAN3	((unsigned short) (card_address+3))
#define B2201_CHAN4	((unsigned short) (card_address+4))
#define B2201_CHAN5	((unsigned short) (card_address+5))
#define B2201_CHAN6	((unsigned short) (card_address+6))
#define B2201_CHAN7	((unsigned short) (card_address+7))

#define ADDR_LOW	(B2201_CHAN0)
#define ADDR_HIGH	(B2201_CHAN2)
#define DATA_CHAN	(B2201_CHAN1)
#define CONTROL		(B2201_CHAN3)

                                // Bit 0 ~Write
#define CONTROL_N_WR       0x01
                                // Bit 1 ~Read
#define CONTROL_N_RD       0x02
                                // Bit 2 ~Chip Select / ~Memory Request
#define CONTROL_N_MREQ     0x04
                                // Bit 3 ~Reset
#define CONTROL_N_RS       0x08
                                // Bit 4 ~Write Flash ROM
#define CONTROL_N_WR_FLASH 0x10
                                // Bit 6 ~LED
#define CONTROL_N_LED      0x40
                                // Bit 7 ~Data Write
#define CONTROL_N_DW       0x80

#define LED_OFF         SET_BIT(CONTROL_N_LED)
#define LED_ON          CLR_BIT(CONTROL_N_LED)

/* Bit routines */
void CLR_BIT(BYTE bit)
{
   card_control_byte &= ~bit;
   OUTPORT(CONTROL, card_control_byte);
}

void SET_BIT(BYTE bit)
{
   card_control_byte |= bit;
   OUTPORT(CONTROL, card_control_byte);
}

/* Data port direction routines */
void DATA_PORT_DIR_IN(void)
{
   SET_BIT(CONTROL_N_DW);
}

void DATA_PORT_DIR_OUT(void)
{
   CLR_BIT(CONTROL_N_DW);
}

/* Read data from RAM(1) or ROM(0) */
BYTE READ(BYTE Mode)
{
   BYTE val;

   LED_ON;
   OUTPORT(ADDR_LOW, card_buf_addr_low);
   OUTPORT(ADDR_HIGH, card_buf_addr_high);
   if(Mode == RAM)
      CLR_BIT(CONTROL_N_MREQ);
   CLR_BIT(CONTROL_N_RD);
   val = INPORT(DATA_CHAN);
   SET_BIT(CONTROL_N_RD);
   if(Mode == RAM)
      SET_BIT(CONTROL_N_MREQ);
   return(val);
}

/* Write data in bank-switch IC */
void WRITE_BKSW(data, address)
{
   LED_ON;
   DATA_PORT_DIR_OUT();
   OUTPORT(DATA_CHAN, (BYTE) data);
   OUTPORT(ADDR_LOW, (BYTE)(address & 0x00FF));
   OUTPORT(ADDR_HIGH, (BYTE)(address >> 8));
   CLR_BIT(CONTROL_N_WR);
   SET_BIT(CONTROL_N_WR);
   DATA_PORT_DIR_IN();
}

/* Write data in RAM */
void WRITE_MEM(val)
{
   LED_ON;
   DATA_PORT_DIR_OUT();
   OUTPORT(DATA_CHAN, (BYTE) val);
   OUTPORT(ADDR_LOW, card_buf_addr_low);
   OUTPORT(ADDR_HIGH, card_buf_addr_high);
   CLR_BIT(CONTROL_N_MREQ);
   CLR_BIT(CONTROL_N_WR);
   SET_BIT(CONTROL_N_WR);
   SET_BIT(CONTROL_N_MREQ);
   DATA_PORT_DIR_IN();
}

/* Write data in Flash IC */
void WRITE_FLASH(val)
{
   LED_ON;
   DATA_PORT_DIR_OUT();
   OUTPORT(DATA_CHAN, (BYTE) val);
   OUTPORT(ADDR_LOW, card_buf_addr_low);
   OUTPORT(ADDR_HIGH, card_buf_addr_high);
   CLR_BIT(CONTROL_N_WR_FLASH);
   SET_BIT(CONTROL_N_WR_FLASH);
   DATA_PORT_DIR_IN();
}
    
/* Write data in Nintendo Flash IC */
void WRITE_NFLASH(val)
{
   LED_ON;
   DATA_PORT_DIR_OUT();
   OUTPORT(DATA_CHAN, (BYTE) val);
   OUTPORT(ADDR_LOW, card_buf_addr_low);
   OUTPORT(ADDR_HIGH, card_buf_addr_high);
   CLR_BIT(CONTROL_N_WR);
   SET_BIT(CONTROL_N_WR);
   DATA_PORT_DIR_IN();
}
    
    
/* write low address byte */
void ADDRESS_LOW(val)
{
   card_buf_addr_low = (BYTE) val;
}

/* write high address byte */
void ADDRESS_HIGH(val)
{
   card_buf_addr_high = (BYTE) val;
}
   
/* usage of hardware */
void usage_hardware(void)
{
   printf("\n");
   printf("The output driver configuration:\n");
   printf("   Port0 U12 74x273 ADR_LOW\n");
   printf("   Port1 U4  74x374 DATA\n");
   printf("   Port2 U3  74x273 ADR_HIGH\n");
   printf("   Port3 U9  74x273 CONTROL\n");
   printf("     Port3 Bit 0 - Not Write                  (Cart Pin  3, Not Write)\n");
   printf("     Port3 Bit 1 - Not Read                   (Cart Pin  4, Not Read)\n");
   printf("     Port3 Bit 2 - Not MREQ / Not Chip Select (Cart Pin  5, Not MREQ)\n");
   printf("     Port3 Bit 3 - Not Reset                  (Cart Pin 30, Not Reset)\n");
   printf("     Port3 Bit 4 - Not Write Flash            (Cart Pin 31, Audio In)\n");
   printf("     Port3 Bit 6 - Not LED\n");
   printf("     Port3 Bit 7 - Not Data Write             (U4 Pin1 74x374\n");
   printf("\n");     
   printf("press any key to continue ...\n");
   getch();
   printf("\t-l\tSpecify the AR B2201 cardaddress (default is 1=0x300, 0=0x310)\n");
}   

/* init hardware */
void init_port(int port)
{
   if (port == 1)
      card_address = 0x300;
   else
      card_address = 0x310;
   if(port == 0)
      printf("Address of AR B2201 I/O card = %hx hex\n",card_address);
   // init control byte
   card_control_byte = 0xFF;
   OUTPORT(CONTROL, card_control_byte);
   // set data port direction
   DATA_PORT_DIR_IN();
   // initialize output values
   OUTPORT(ADDR_LOW, 0x00);
   OUTPORT(ADDR_HIGH, 0x00);
   OUTPORT(DATA_CHAN, 0x00);
   // perform reset
   CLR_BIT(CONTROL_N_RS);
   SET_BIT(CONTROL_N_RS);
   LED_OFF;
}

/* testprogram for AR B2201 */
void test()
{
   LED_ON;
   OUTPORT(ADDR_LOW, 0x00);
   OUTPORT(ADDR_HIGH, 0x00);

   // DATA lines
   printf("\nD0 - D7 data lines check !\n");
   printf("Check pins <22 up to 29> of GB connector\n");
   DATA_PORT_DIR_OUT();
   OUTPORT(DATA_CHAN, 0x00);
   printf("They should all be low. Press <RETURN>\n");
   getch();

   OUTPORT(DATA_CHAN, 0xFF);
   printf("Now, they should all be high. Press <RETURN>\n");
   getch();
   DATA_PORT_DIR_IN();

   // ADDR LOW lines
   printf("\n\n\nA0 - A7 address lines check !\n");
   printf("Check pins <6 up to 13> of GB connector\n");
   OUTPORT(ADDR_LOW, 0x00);
   printf("They should all be low. Press <RETURN>\n");
   getch();
   	
   OUTPORT(ADDR_LOW, 0xFF);
   printf("Now, they should all be high. Press <RETURN>\n");
   getch();

   // ADDR HIGH lines
   printf("\n\n\nA8 - A15 address lines check !\n");
   printf("Check pins <14 up to 21> of GB connector\n");
   OUTPORT(ADDR_HIGH, 0x00);
   printf("They should all be low. Press <RETURN>\n");
   getch();

   OUTPORT(ADDR_HIGH, 0xFF);
   printf("Now, they should all be high. Press <RETURN>\n");
   getch();

   // WR line
   printf("\n\n\nWR line check !\n");
   printf("check pin <3> of GB connector\n");
   CLR_BIT(CONTROL_N_WR);
   printf("Should be low. Press <RETURN>\n");
   getch();

   SET_BIT(CONTROL_N_WR);
   printf("Should be high. Press <RETURN>\n");
   getch();

   // RD line
   printf("\n\n\nRD line check !\n");
   printf("check pin <4> of GB connector\n");
   CLR_BIT(CONTROL_N_RD);
   printf("Should be low. Press <RETURN>\n");
   getch();

   SET_BIT(CONTROL_N_RD);
   printf("Should be high. Press <RETURN>\n");
   getch();

   // CS / MREQ line
   printf("\n\n\nCS line check !\n");
   printf("check pin <5> of GB connector\n");
   CLR_BIT(CONTROL_N_MREQ);
   printf("Should be low. Press <RETURN>\n");
   getch();

   SET_BIT(CONTROL_N_MREQ);
   printf("Should be high. Press <RETURN>\n");
   getch();

   // RS line
   printf("\n\n\nRS line check !\n");
   printf("check pin <30> of GB connector\n");
   CLR_BIT(CONTROL_N_RS);
   printf("Should be low. Press <RETURN>\n");
   getch();

   SET_BIT(CONTROL_N_RS);
   printf("Should be high. Press <RETURN>\n");
   getch();

   // Audio IN line
   printf("\n\n\nAudio IN line check !\n");
   printf("check pin <31> of GB connector\n");
   CLR_BIT(CONTROL_N_WR_FLASH);
   printf("Should be low. Press <RETURN>\n");
   getch();

   SET_BIT(CONTROL_N_WR_FLASH);
   printf("Should be high. Press <RETURN>\n");
   getch();

   LED_OFF;
   printf("\n\n\nTest complete !\n\n");
}