/************************************************/
/*												*/
/*	definition file for Bung hardware V1.2		*/
/*												*/
/*  Nintendo 32MBit carts are not supported !   */
/*  add support for SPP                         */
/*												*/
/************************************************/

unsigned port_8;
unsigned port_9;
unsigned port_a;
unsigned port_b;
unsigned port_c;      
unsigned data_port;   
char 	 port_type=0;       // 0=epp, 1=spp
char 	 epp_spp=0;      
int      BungHW = 0;        // Flag for Bung Xchanger

#define ai              port_b
#define data            port_c

#define set_ai_write    OUTPORT(port_a,5)	// 5 ninit=1, nwrite=0
#define set_data_write  OUTPORT(port_a,1)	// 1 ninit=0, nwrite=0
#define set_data_read   OUTPORT(port_a,0)	// 0 ninit=0, nwrite=1
#define set_normal      OUTPORT(port_a,4)	// 4 ninit=1, nwrite=1

void spp_set_ai(unsigned char _ai)
{
   set_data_write;
   OUTPORT(port_8,_ai);        // put ai at data bus
   OUTPORT(port_a,9);          // nastb=0,nib_sel=0,ndstb=1,nwrite=0
   OUTPORT(port_a,1);          // nastb=1,nib_sel=0,ndstb=1,nwrite=0
                               // nastb ~~~~|___|~~~~
}

void spp_write_data(unsigned char _data)
{
   OUTPORT(port_8,_data);      // put data at data bus
   OUTPORT(port_a,3);          // nastb=1,nib_sel=0,ndstb=0,nwrite=0
   OUTPORT(port_a,1);          // nastb=1,nib_sel=0,ndstb=1,nwrite=0
                               // ndstb ~~~~|___|~~~~
}

void spp_set_ai_data(unsigned char _ai,unsigned char _data)
{
   spp_set_ai(_ai);
   spp_write_data(_data);
}

char spp_read_data(void)
{
   char low_nibble,high_nibble,temp;
   
   set_data_read;
   OUTPORT(port_a,2);           // nastb=1,nib_sel=0,ndstb=0,nwrite=1
   low_nibble=INPORT(port_9);
   OUTPORT(port_a,6);           // nastb=1,nib_sel=1,ndstb=0,nwrite=1
   high_nibble=INPORT(port_9);
   OUTPORT(port_a,0);           // nastb=1,nib_sel=0,ndstb=1,nwrite=1
                                // nibble_sel ___|~~~ and ndstb ~~~~|___|~~~~
   temp=(((high_nibble<<1)&0xf0)|((low_nibble>>3)&0x0f));
   return(temp);
}

void epp_set_ai(unsigned char _ai)
{
   set_data_write;			
   OUTPORT(ai,_ai);
}

void epp_set_ai_data(unsigned char _ai,unsigned char _data)
{
   epp_set_ai(_ai);
   set_data_write;			// ninit=0, nwrite=0
   OUTPORT(data,_data);
}

void set_ai(unsigned char _ai)
{
   if (port_type)
        spp_set_ai(_ai);
   else
        epp_set_ai(_ai);
}

void set_ai_data(unsigned char _ai,unsigned char _data)
{
   if (port_type)
        spp_set_ai_data(_ai,_data);     // spp mode
   else
        epp_set_ai_data(_ai,_data);     // epp mode
}

/* disable LED */
#define LED_OFF         set_ai_data(2,0); set_normal			// ninit=1, nWrite=1

/* read data from RAM(1) or ROM(0) */
/* selection will be done by hardware */
BYTE READ(BYTE Mode)
{   
   	set_ai(3);				// default write mode
   	set_data_read;			// ninit=0, nWrite=1
    if (port_type){
        return(spp_read_data());        // spp read data
    }
    else {
        return(INPORT(data));           // epp read data
    }
}                           

/* write data in bank-switch IC */
void WRITE_BKSW(_data, _address) 
{  
	set_ai_data(2,0x80);
    set_ai_data(0,(BYTE)(_address & 0x00FF));	
    set_ai_data(1,(BYTE)(_address >> 8));	
    set_ai_data(3,(BYTE)(_data));	
}

/* write data in RAM */
void WRITE_MEM(val)
{
	set_ai_data(2,0x80);
   	set_ai(3);
   	set_data_write;
    if (port_type)
        spp_write_data((BYTE)val);// spp write data
    else
        OUTPORT(data,val);  // epp write data
}

/* write data in Flash IC */
void WRITE_FLASH(val)
{
   	set_ai_data(2,0x82);	// wei enable
   	set_ai(3);				// default write mode
   	set_data_write;
    if (port_type)
        spp_write_data((BYTE)val);// spp write data
    else
        OUTPORT(data,val);  // epp write data}
   	set_ai_data(2,0x80);	// wei disable   
}

/* write data in Nintendo flash cart */
void WRITE_NFLASH(val)
{
	printf("  Not supported by Bung GB-Xchanger !\n");
	LED_OFF;
	exit(1); 
}

/* write low address byte */
void ADDRESS_LOW(val)
{
    set_ai_data(0,(BYTE)(val));	
}

/* write high address byte */
void ADDRESS_HIGH(val)
{
    set_ai_data(1,(BYTE)(val));	
}
                            
BYTE check_port(void)
{  
   set_ai_data(1,0x12);
   set_ai_data(0,0x34);
   set_ai(1);
   set_data_read;			// ninit=0, nwrite=1    
   
   if (port_type){ if (spp_read_data() != 0x12) return(1); }
   else { if (INPORT(data) != 0x12) return(1); }             
   
   set_ai(0);
   set_data_read;			// ninit=0, nwrite=1
   
   if (port_type){ if (spp_read_data() != 0x34) return(1); }
   else { if (INPORT(data) != 0x34) return(1); }

   LED_OFF;
   return(0);
}    

/* usage of hardware */     
void usage_hardware(void)
{
	printf("\t-l\tSpecify the SPP/EPP port to use (default is 1 = LPT1)\n");
}	

/* init port */
void init_port(int port)
{                                                                      
	data_port = *(unsigned far *)MK_FP(0x0040, 6 + 2*port);
	if(data_port == 0)
	{
		printf("Can't find address of SPP/EPP parallel port %d...\n", port);
		exit(1);
	} else {
		port_8 = data_port;
		port_9 = data_port + 1;
		port_a = data_port + 2;
		port_b = data_port + 3;
		port_c = data_port + 4;
   		OUTPORT(port_9,1);		// clear EPP time flag
   		set_ai_data(2,0);		// rst=0, wei=0(dis.), rdi=0(dis.)    
   		set_ai_data(2,0x80);	// rst=1, wei=0(dis.), rdi=0(dis.)  

   		port_type=1;         	// 0=epp, 1=spp
   		if (check_port()==0){
      		epp_spp=1;          // epp port present
   		}
   		port_type=0;            // 0=epp, 1=spp
   		if (check_port()){
      		if (epp_spp){
         		port_type=1;
       		}
      		else { printf("\nNo GB-Xchanger present !\07\n\n"); exit(1); }
   		}
        printf("GB-Xchanger found at port %hX\n\n",port_8);
  
   		OUTPORT(port_9,1);		// clear EPP time flag
   		set_ai_data(2,0);		// rst=0, wei=0(dis.), rdi=0(dis.)    
   		set_ai_data(2,0x80);	// rst=1, wei=0(dis.), rdi=0(dis.)  
	}
}

/* testprogram for EPLD hardware */
void test()
{
        printf("\n\n\nno Testprocedure implemented !\n\n");
}